"""
SQLAlchemy models for hotcooler
"""

from uuid import uuid4

import sqlalchemy as sa
from sqlalchemy import orm
from sqlalchemy.dialects.postgresql import UUID

from websauna.system.model.meta import Base
from websauna.system.model.columns import UTCDateTime


class Client(Base):
    """
    Represents a tempmon client
    """
    __tablename__ = 'client'

    id = sa.Column(sa.Integer(), autoincrement=True, primary_key=True)
    uuid = sa.Column(UUID(as_uuid=True), default=uuid4)

    config_key = sa.Column(sa.String(50), nullable=False, unique=True)
    hostname = sa.Column(sa.String(255), nullable=False)
    location = sa.Column(sa.String(255), nullable=True)

    delay = sa.Column(sa.Integer(), nullable=True, doc="""
    Number of seconds to delay between reading / recording temperatures.  If
    not set, a default of 60 seconds will be assumed.
    """)

    enabled = sa.Column(sa.Boolean(), nullable=False, default=False)
    online = sa.Column(sa.Boolean(), nullable=False, default=False)

    probes = orm.relationship('Probe',
                              back_populates='client',
                              cascade='all, delete-orphan',
                              single_parent=True)

    def __str__(self):
        return '{} ({})'.format(self.config_key, self.hostname)

    def enabled_probes(self):
        return [probe for probe in self.probes if probe.enabled]


class Probe(Base):
    """
    Represents a probe connected to a tempmon client
    """
    __tablename__ = 'probe'

    id = sa.Column(sa.Integer(), autoincrement=True, primary_key=True)
    uuid = sa.Column(UUID(as_uuid=True), default=uuid4)

    client_id = sa.Column(sa.Integer(), sa.ForeignKey('client.id'))
    client = orm.relationship(Client, back_populates='probes')

    config_key = sa.Column(sa.String(length=50), nullable=False, unique=True)
    appliance_type = sa.Column(sa.Integer(), nullable=False)
    description = sa.Column(sa.String(length=255), nullable=False)
    device_path = sa.Column(sa.String(length=255), nullable=True)
    enabled = sa.Column(sa.Boolean(), nullable=False, default=True)

    good_temp_min = sa.Column(sa.Integer(), nullable=False)
    good_temp_max = sa.Column(sa.Integer(), nullable=False)
    critical_temp_min = sa.Column(sa.Integer(), nullable=False)
    critical_temp_max = sa.Column(sa.Integer(), nullable=False)
    therm_status_timeout = sa.Column(sa.Integer(), nullable=False)
    status_alert_timeout = sa.Column(sa.Integer(), nullable=False)

    status = sa.Column(sa.Integer(), nullable=True)
    status_changed = sa.Column(UTCDateTime, nullable=True)
    status_alert_sent = sa.Column(UTCDateTime, nullable=True)

    def __str__(self):
        return self.description


class Reading(Base):
    """
    Represents a single temperature reading from a tempmon probe
    """
    __tablename__ = 'reading'

    id = sa.Column(sa.Integer(), autoincrement=True, primary_key=True)
    uuid = sa.Column(UUID(as_uuid=True), default=uuid4)

    client_id = sa.Column(sa.Integer(), sa.ForeignKey('client.id'))
    client = orm.relationship(Client)

    probe_id = sa.Column(sa.Integer(), sa.ForeignKey('probe.id'))
    probe = orm.relationship(Probe)

    taken = sa.Column(UTCDateTime, default=None)
    degrees_f = sa.Column(sa.Numeric(precision=7, scale=4), nullable=False)

    def __str__(self):
        return str(self.degrees_f)
